/*

    Zelda Picross

    Copyright (C) 2010  Vincent Jouillat

    Please send bugreports with examples or suggestions to www.zeldaroth.fr

*/

#include <sstream>
#include <fstream>
#include <iostream>

#include <SDL/SDL.h>

#include "Audio.h"

Audio Audio::instance=Audio();

Audio::Audio() : musiqueId(0), playing(false) {
    SOUND = true;
    music = NULL;

    if(SDL_InitSubSystem(SDL_INIT_AUDIO) == -1) SOUND = false;

    if (SOUND) {
        Mix_OpenAudio(44100, AUDIO_S16SYS, 2, 2048);
        previous_volume = Mix_VolumeMusic(32);
        loadSounds();
        setVolson(32);
    }
}

Audio::~Audio() {
    if (SOUND) {
        freeSounds();
	Mix_PauseMusic();
        Mix_VolumeMusic(previous_volume);
        Mix_HaltMusic();
        Mix_FreeMusic(music);
        Mix_CloseAudio();
    }
}

Audio* Audio::getInstance() {
    return &instance;
}

void Audio::setVolume(int volume) {
        Mix_VolumeMusic(volume);
}

void Audio::setVolson(int volson) {
    for (int i = 0; i < 16; i++)
                Mix_VolumeChunk(sons[i], volson);
}

void Audio::loadSounds() {
    sons = new Mix_Chunk*[16];
    
    sons[0] = Mix_LoadWAV("data/sound/text.ogg"); // lettres
    sons[1] = Mix_LoadWAV("data/sound/menu1.ogg"); // menu 1
    sons[2] = Mix_LoadWAV("data/sound/menu2.ogg"); // menu 2
    sons[3] = Mix_LoadWAV("data/sound/menu3.ogg"); // menu 3
    sons[4] = Mix_LoadWAV("data/sound/surprise.ogg"); // surprise
    sons[5] = Mix_LoadWAV("data/sound/monte.ogg"); // monte
    sons[6] = Mix_LoadWAV("data/sound/descend.ogg"); // descend
    sons[7] = Mix_LoadWAV("data/sound/bomb.ogg"); // bombe
    sons[8] = Mix_LoadWAV("data/sound/textnext.ogg"); // suite texte
    sons[9] = Mix_LoadWAV("data/sound/textend.ogg"); // fin texte
    sons[10] = Mix_LoadWAV("data/sound/happy.ogg"); // trouve objet
    sons[11] = Mix_LoadWAV("data/sound/sword.ogg"); // pe
    sons[12] = Mix_LoadWAV("data/sound/Sword360.ogg"); // spin
    sons[13] = Mix_LoadWAV("data/sound/stamp.ogg"); // pose bombe
    sons[14] = Mix_LoadWAV("data/sound/danger.ogg"); // danger
    sons[15] = Mix_LoadWAV("data/sound/hurt.ogg"); // link se blesse
    
}

void Audio::freeSounds() {
    if (SOUND) {
        for (int i = 0; i < 16; i++) Mix_FreeChunk(sons[i]);
        delete[] sons;
    }
}

void Audio::playSound(int id, int chl) {
    if (SOUND) Mix_PlayChannel(chl, sons[id],0);
}

void Audio::stopSound() {
    if (SOUND) {Mix_HaltChannel(-1); musiqueId=0;}
}

void Audio::playMusic(int id) {
    if (SOUND) {
        if (musiqueId != id || !playing) {
            musiqueId = id;
            Mix_HaltMusic();
            Mix_FreeMusic(music);
            music = choixMusique(id);
	    Mix_PlayMusic(music,-1);
            playing = true;
        }
    }
}

void Audio::stopMusic() {
    if (SOUND) Mix_HaltMusic();
    playing = false;
}

void Audio::replayMusic() {
    if (SOUND) Mix_PlayMusic(music,-1);
    playing = true;
}

Mix_Music* Audio::choixMusique(int id) {
    switch (id) {
        case 1 : return Mix_LoadMUS("data/music/Plaine.mid");
        case 2 : return Mix_LoadMUS("data/music/Home.mid");
        case 3 : return Mix_LoadMUS("data/music/Lac.mid");
        case 4 : return Mix_LoadMUS("data/music/Cocorico.mid");
        case 5 : return Mix_LoadMUS("data/music/Mont.mid");
        case 6 : return Mix_LoadMUS("data/music/BoisPerdus.mid");
        case 7 : return Mix_LoadMUS("data/music/Jeu.mid");
        case 8 : return Mix_LoadMUS("data/music/Chateau.mid");
        case 9 : return Mix_LoadMUS("data/music/Boss.mid");
        case 10 : return Mix_LoadMUS("data/music/DFinal.mid");
        case 11 : return Mix_LoadMUS("data/music/BossF.mid");
        case 12 : return Mix_LoadMUS("data/music/Zelda.mid");
        case 190 : return Mix_LoadMUS("data/music/Selection.mid");
        case 195 : return Mix_LoadMUS("data/music/Magasin.mid");
        case 200 : return Mix_LoadMUS("data/music/Titre.mid");
        case 210 : return Mix_LoadMUS("data/music/Debut.mid");
        case 220 : return Mix_LoadMUS("data/music/Mort.mid");
        case 230 : return Mix_LoadMUS("data/music/Fin.mid");
        default : return Mix_LoadMUS("data/music/Plaine.mid");
    }
}
